/***************************************************************************
 *   Copyright (C) 2006 by Jeziorski, Weintraub, Benkard and Van Roy       *
 *   przemekj@stanford.edu                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "compoe_nonstat.h"
CompOE_nonstat::CompOE_nonstat() {
  xmax = 0;
  ignore = 2;
  rhoMatrix_global=NULL;
  iotaMatrix_global=NULL;
  lambdaVector_global=NULL; 
  V_global=NULL; 
  s_global=NULL;
  p_global=NULL;
  profitMatrix_global=NULL;
  bound1avg_nonstat=NULL;
  bound1var_nonstat=NULL;
  bound1pre_nonstat=NULL;
  bound1por_nonstat=NULL;
  bound2avg_nonstat=NULL;
  bound2var_nonstat=NULL;
  bound2pre_nonstat=NULL;
  bound2por_nonstat=NULL;
  bound3avg_nonstat=NULL;
  bound3var_nonstat=NULL;
  bound3pre_nonstat=NULL;
  bound3por_nonstat=NULL;
}

void CompOE_nonstat::tloop(MatrixMTL<double> &rhoMatrix, MatrixMTL<double> &iotaMatrix, MatrixMTL<double> &pMatrix,
  VectorMTL<double> &lambda1, MatrixMTL<double> &VMatrix, MatrixMTL<double> &s, 
  double lambda_oe, VectorMTL<double> &d, double &Delta0, double &Delta1) {

  VectorMTL<double> iota(xmax);
  VectorMTL<double> rho(xmax);
  VectorMTL<double> prcont(xmax);
  VectorMTL<double> condexp(xmax);
  VectorMTL<double> profit(xmax);
  VectorMTL<double> V(xmax);
  VectorMTL<double> temp(xmax);

  #if TRANSITION_MATRIX == 0 
    MatrixMTL<double> tranmat(xmax);
    MatrixMTL<double> conttranmat(xmax);
  #elif TRANSITION_MATRIX == 1 
    TriDiagMatrixMTL<double> tranmat(xmax);
    TriDiagMatrixMTL<double> conttranmat(xmax);
  #elif TRANSITION_MATRIX == 2 
    SparseMatrixMTL<double> tranmat;
    SparseMatrixMTL<double> conttranmat;
  #else 
    #error Invalid value for TRANSITION_MATRIX
  #endif
  V = VMatrix.getRow(Tbar+1);

  prcont = 0;

  for(int i=Tbar; i>=0; i--) {
//    cout << "Tloop for t = " << i << endl;
    /*** Optimal investment ***/
    iota = optInv(V,d);
    iotaMatrix.setRow(i,iota);


    /*** Compute transition probabilities ***/
    tranProb(iota,tranmat,conttranmat,prcont);

    /*** Compute continuation values if not exiting ***/
    rho = (-iota*d) + ((tranmat.dot(V))*constants[2]);
    rhoMatrix.setRow(i,rho);

    /*** Compute continuation probs. and conditional expectation sell-off ***
     *** value, conditional on exiting                                    ***/
    continuationProb(rho,prcont,condexp);
    condToNoncondTranProb(tranmat,conttranmat,prcont);

    temp = s.getRow(i);

    VectorMTL<double> p0(xmax);
    p0 = pMatrix.getRow(i);
    compProf(temp,p0,profit);
    pMatrix.setRow(i,p0);
    profitMatrix_global->setRow(i,profit);

    VectorMTL<double> Vold(xmax);
    Vold=V;

    V = profit + (-prcont+1.0)*condexp+prcont*rho;

    VMatrix.setRow(i,V);

    /*** Fixed number of firms ***/
    if(control[2]>0) {
      Delta0=0;
      Delta1=0;

      lambda1[i]=control[2]-1;
    } else { // Entry/exit
      /*** Update Delta ***/
      Delta0 = MAX(Delta0,(Vold[int(constants[3])-1]*constants[2]-constants[0]));
      if(lambda1[i]>nonstatparams[15]) {
        Delta1 = MAX(Delta1,-(Vold[int(constants[3])-1]*constants[2]-constants[0]));
      }

      /*** Set lambda_{Tbar}=lambda_oe ***/
      if(i==Tbar) {
        lambda1[i]=lambda_oe;
      } else {
//      cout << "Update lambda: " << Vold[int(constants[3])-1]*constants[2]/constants[0] << endl;
        lambda1[i]*=Vold[int(constants[3])-1]*constants[2]/constants[0];
      }
    }
  }
}
void CompOE_nonstat::computeState(MatrixMTL<double> &iotaMatrix, MatrixMTL<double> &rhoMatrix,
  VectorMTL<double> &lambda1, MatrixMTL<double> &s) {

#if TRANSITION_MATRIX == 0 
  MatrixMTL<double> tranmat(xmax,xmax);
  MatrixMTL<double> conttranmat(xmax,xmax);
#elif TRANSITION_MATRIX == 1 
  TriDiagMatrixMTL<double> tranmat(xmax);
  TriDiagMatrixMTL<double> conttranmat(xmax);
#elif TRANSITION_MATRIX == 2 
  SparseMatrixMTL<double> tranmat;
  SparseMatrixMTL<double> conttranmat;
#else 
  #error Invalid value for TRANSITION_MATRIX
#endif

  VectorMTL<double> temp(xmax);
  VectorMTL<double> contprob(xmax);
  VectorMTL<double> rho(xmax);
  VectorMTL<double> iota(xmax);
  for(int t=0;t<Tbar;t++) {
    iota = iotaMatrix.getRow(t);
    rho = rhoMatrix.getRow(t);
    continuationProb(rho,contprob,temp);
    tranProb(iota,tranmat,conttranmat,contprob);
    temp = conttranmat.primedot(s.getRow(t));
    if(control[2]==0)
      temp[int(constants[3])-1]+=lambda1[t];
    s.setRow(t+1,temp);
  }
}

void CompOE_nonstat::main_loop(int xmax_init, int TbarInit, VectorMTL<double> &VInit, 
  VectorMTL<double> &iotaInit, VectorMTL<double> &rhoInit, VectorMTL<double> &s0_init, VectorMTL<double> &tildes,
  VectorMTL<double> &prices_init ,double lambda_oe) {

  // We want Tbar + 1 periods
  Tbar=TbarInit;
  xmax = xmax_init;

  MatrixMTL<double> rhoMatrix0(Tbar+1,xmax); 
  MatrixMTL<double> iotaMatrix0(Tbar+1,xmax);
  VectorMTL<double> lambda0(Tbar+1);
  MatrixMTL<double> rhoMatrix1(Tbar+1,xmax); 
  MatrixMTL<double> iotaMatrix1(Tbar+1,xmax);
  VectorMTL<double> lambda1(Tbar+1);

  VectorMTL<double> contprob(xmax);
  VectorMTL<double> iota0(xmax);
  VectorMTL<double> rho0(xmax);

  #if TRANSITION_MATRIX == 0 
    MatrixMTL<double> tranmat(xmax,xmax);
    MatrixMTL<double> conttranmat(xmax,xmax);
  #elif TRANSITION_MATRIX == 1 
    TriDiagMatrixMTL<double> tranmat(xmax);
    TriDiagMatrixMTL<double> conttranmat(xmax);
  #elif TRANSITION_MATRIX == 2 
    SparseMatrixMTL<double> tranmat;
    SparseMatrixMTL<double> conttranmat;
  #else 
    #error Invalid value for TRANSITION_MATRIX
  #endif

  MatrixMTL<double> VMatrix(Tbar+2,xmax);
  MatrixMTL<double> s(Tbar+2,xmax);
  VectorMTL<double> V(xmax);
  VectorMTL<double> temp(xmax);
  VectorMTL<double> condexp(xmax);

  // Set last expected state to OE
  s.setRow(Tbar+1,tildes);

  MatrixMTL<double> contprobMatrix0(Tbar,xmax),contprobMatrix1(Tbar,xmax);
  contprobMatrix1=0;

  /* Initialize profit matrix */
  if(profitMatrix_global!=NULL) {
    delete profitMatrix_global;
  }
  profitMatrix_global = new MatrixMTL<double>(Tbar+1,xmax);

  /* Precompute investment cost */
  VectorMTL<double> d(xmax);
  for(int i=0;i<xmax;i++) {
    d[i]=MACRO__MINVCOST(i);
  }

  /* Initialize the startup prices */
  MatrixMTL<double> pMatrix(Tbar+1,xmax);

  struct tms t,u;
  long r1,r2;

  /*** Innitialize states ***/
  s.setRow(0,s0_init);

  /*** Innitialize calulations ***/
  iota0 = iotaInit;
  rho0 = rhoInit;
  for(int i=0;i<=Tbar;i++) {
    rhoMatrix0.setRow(i,rhoInit);
    iotaMatrix0.setRow(i,iotaInit);
    pMatrix.setRow(i,prices_init);
  }

  lambda0 = lambda_oe;

  int n=1;
  for(ignore=int(nonstatparams[14]);ignore<3;ignore++) {
//  for(ignore=2;ignore<3;ignore++) {
     n=1;
     if(ignore==1) { 
      #ifdef COUT_DEBUG
        cout << "Nonstationary: Compute with approximated profit function"<< endl;
      #endif
      #ifdef DEBUG
        *myfile << "Nonstationary: Compute with approximated profit function"<< endl;
      #endif
    } else {
      #ifdef COUT_DEBUG
        cout << "Nonstationary: Compute with full profit function"<< endl;
      #endif
      #ifdef DEBUG
        *myfile << "Nonstationary: Compute with full profit function"<< endl;
      #endif
    }
    while(1) {
      r1 = times(&t);
      #ifdef COUT_DEBUG
        cout << "\nNonstationary: iteration #" << n <<endl;
      #endif
      #ifdef DEBUG
        *myfile << "\nNonstationary: iteration #" << n <<endl;
      #endif

      lambda1=lambda0;

      /*** Reset tolerance for lambda ***/
      double Delta0=0, Delta1=0;

      /*** Update continuation probability matrix ***/
      contprobMatrix0=contprobMatrix1;

      computeState(iotaMatrix0,rhoMatrix0,lambda1,s);

      /*** Init. V_{Tbar+1} ***/
      VMatrix.setRow(Tbar+1,VInit);
      tloop(rhoMatrix1, iotaMatrix1, pMatrix, lambda1, VMatrix, s, lambda_oe, d, Delta0, Delta1);
      r2 = times(&u);


      #ifdef COUT_DEBUG
        cout << "Iota stop: " << norm(iotaMatrix0-iotaMatrix1) << "\t\t Rho stop = "
          << norm(rhoMatrix0-rhoMatrix1) << endl;
        cout << "Lambda stop: Delta0 = " << Delta0 << ", Delta1 = " 
          << Delta1 << ", norm = " << norm(lambda1-lambda0)<< endl;
        cout << "Time of the iteration: " << r2-r1 <<endl;
      #endif

      #ifdef DEBUG
        *myfile << "Iota stop: " << norm(iotaMatrix0-iotaMatrix1) << "\t\t Rho stop = "
          << norm(rhoMatrix0-rhoMatrix1) << endl;
        *myfile << "Lambda stop: Delta0 = " << Delta0 << ", Delta1 = "<< Delta1 <<  ", norm = " 
          << norm(lambda1-lambda0)<< endl;
        *myfile << "Time of the iteration: " << r2-r1 <<endl;
      #endif

      /*** Stopping criterion ***/
      if ((norm(iotaMatrix0-iotaMatrix1) < nonstatparams[1]) && 
        (norm(rhoMatrix0-rhoMatrix1) < nonstatparams[0]) &&
        (Delta0<nonstatparams[2]) && (Delta1<nonstatparams[2])) { 

        break;
      }

      #ifdef COUT_DEBUG
        cout << "Update steps: " << (1/(pow(n,nonstatparams[4])+nonstatparams[6])) << " " 
          << (1/(pow(n,nonstatparams[3])+nonstatparams[5])) << endl;
      #endif

      #ifdef DEBUG
        *myfile << "Update steps: " << (1/(pow(n,nonstatparams[4])+nonstatparams[6])) << " " 
          << (1/(pow(n,nonstatparams[3])+nonstatparams[5])) << endl;
      #endif

      lambda0+=(lambda1-lambda0)/(pow(n,nonstatparams[4])+nonstatparams[6]);
      rhoMatrix0+=(rhoMatrix1-rhoMatrix0)/(pow(n,nonstatparams[3])+nonstatparams[5]);
      iotaMatrix0+=(iotaMatrix1-iotaMatrix0)/(pow(n,nonstatparams[3])+nonstatparams[5]);

      n++;
//      cout << s.getRow(Tbar+1); getchar();
    }
  }

  if(rhoMatrix_global!=NULL) {
    delete rhoMatrix_global;
  }
  rhoMatrix_global = new MatrixMTL<double>(Tbar+1,xmax);
  *rhoMatrix_global = rhoMatrix1;

  if(iotaMatrix_global!=NULL) {
    delete iotaMatrix_global;
  }
  iotaMatrix_global = new MatrixMTL<double>(Tbar+1,xmax);
  *iotaMatrix_global = iotaMatrix1;

  if(lambdaVector_global!=NULL) {
    delete lambdaVector_global;
  }
  lambdaVector_global = new VectorMTL<double>(Tbar+1);
  *lambdaVector_global = lambda1;

  if(V_global!=NULL) {
    delete V_global;
  }
  V_global = new MatrixMTL<double>(Tbar+2,xmax);
  *V_global = VMatrix;

  if(s_global!=NULL) {
    delete s_global;
  }
  s_global = new MatrixMTL<double>(Tbar+2,xmax);
  *s_global = s;

  if(p_global!=NULL) {
    delete p_global;
  }
  p_global = new MatrixMTL<double>(Tbar+1,xmax);
  *p_global = pMatrix;

//  cout << *profitMatrix_global;
}
/*************************************************************************
 * This function takes oblivious value function as input and calculates: *
 * a) TbarOut - length of the horizon of non-stationary equilibrium      *
 * b) index - maximum market size                                        *
 * DISABLED                                                              *
 *************************************************************************/
/*
void CompOE_nonstat::computeT(VectorMTL<double> *V,VectorMTL<double> *iotaInit,VectorMTL<double> *rhoInit,double lambdaInit, int &t, int &sizeOfV) {
  sizeOfV = V->getSize();
  CompOE *oe = new CompOE;
  oe->initialize(control, prof_fun, transition, constants, init, convtol);
  oe->setOutput(myfile);

  // Apply multiplier //
  int sizeOfV_old = sizeOfV;
  sizeOfV = int(sizeOfV*nonstatparams[8]);
  for(int i=sizeOfV_old;i<sizeOfV;i++) {
    V->addElement((*V)[sizeOfV_old-1]);
    iotaInit->addElement((*iotaInit)[sizeOfV_old-1]);
    rhoInit->addElement((*rhoInit)[sizeOfV_old-1]);
  }

  while(1) {
    // Check if the size of V is enought to calcutate Tbar //
    t = int(ceil((sizeOfV-constants[3])/nonstatparams[11])); 
    if(t<nonstatparams[12]) {
      sizeOfV=int(constants[3]+nonstatparams[12]*nonstatparams[11]+nonstatparams[9]);
      *myfile << "Current size of V cannot support starting index" <<endl;
      *myfile << "New size: " << sizeOfV << endl << endl;
      #ifdef DEBUG
        cout << "Current size of V cannot support starting index" <<endl;
        cout << "New size: " << sizeOfV << endl << endl;
      #endif
      oe->recompute(*iotaInit,*rhoInit,lambdaInit,sizeOfV);
      V = oe->getV();
    } else if(pow(constants[2],t-nonstatparams[12])*(*V)[sizeOfV-1]>nonstatparams[7]) {
      #ifdef DEBUG
        cout << "Current size of V cannot reach desired precision" <<endl;
        cout << "V(x^e+t w) = " << (*V)[sizeOfV-1] <<endl;
        cout << "xmax = " << sizeOfV << "\t" << "Tbar = " << t << "\t"<< "Epsilon = " << pow(constants[2],t-nonstatparams[12])*(*V)[sizeOfV-1] << endl;
      #endif
      *myfile << "Current size of V cannot reach desired precision" <<endl;
      *myfile << "V(x^e+t w) = " << (*V)[sizeOfV-1] <<endl;
      *myfile << "xmax = " << sizeOfV << "\t" << "Tbar = " << t << "\t"<< "Epsilon = " << pow(constants[2],t-nonstatparams[12])*(*V)[sizeOfV-1] << endl;
//      getchar();
      sizeOfV+=int(nonstatparams[9]);
      #ifdef DEBUG
        cout << "New size: " << sizeOfV << endl << endl;
      #endif
      *myfile << "New size: " << sizeOfV << endl << endl;
      oe->recompute(*iotaInit,*rhoInit,lambdaInit,sizeOfV);
      V = oe->getV();
      iotaInit = oe->getIota();
      rhoInit = oe->getRho();
    } else {
      break;
    }
  }
  delete oe;
}*/

void CompOE_nonstat::computeBoundsNonstat(double *bounds, MatrixMTL<double> &iotaMatrix, MatrixMTL<double> &rhoMatrix,
  MatrixMTL<double> &sMatrix, VectorMTL<double> &lambdaVector, MatrixMTL<double> &pMatrix) {

  xmax = pMatrix.getLength();
  int Tbar = pMatrix.getHeight()-1;

  VectorMTL<double> iota(xmax);
  VectorMTL<double> rho(xmax);

  VectorMTL<double> prcont(xmax);
  VectorMTL<double> condexp(xmax);

  VectorMTL<double> tildes(xmax);
  VectorMTL<double> proftildes(xmax);
  VectorMTL<double> prices(xmax);

  VectorMTL<double> term2(xmax);

  VectorMTL<double> bound1sum(xmax), bound1sqr(xmax), bound1sum_sim(xmax);
  VectorMTL<double> bound2sum(xmax), bound2sqr(xmax), bound2sum_sim(xmax);
  VectorMTL<double> bound3sum(xmax), bound3sqr(xmax), bound3sum_sim(xmax);

  VectorMTL<double> prof_oe(xmax);
  // Determine kmax
  prices = pMatrix.getRow(Tbar);
  tildes = sMatrix.getRow(Tbar+1);
  compProf(tildes, prices, prof_oe);

  int kmax=int(MAX(xmax+1,log(0.01*(1.0-constants[2])/prof_oe[xmax-1])/log(constants[2])));
  kmax=MAX(Tbar+2,kmax); // It must be at least Tbar+2

//  int kmax=100;

  #if TRANSITION_MATRIX == 0 
    MatrixMTL<double> tranMatrix(xmax,xmax);
    MatrixMTL<double> contTranMatrix(xmax,xmax);
    MatrixMTL<double> contTranMatrixOE(xmax,xmax);
    MatrixMTL<double> contTranMatrix_infty(xmax,xmax);
    MatrixMTL<double> contTranMatrix_temp(xmax,xmax);
    MatrixMTL3D<double> prcontMatrix(xmax,xmax,Tbar);
  #elif TRANSITION_MATRIX == 1 
    TriDiagMatrixMTL<double> tranMatrix(xmax);
    TriDiagMatrixMTL<double> contTranMatrix(xmax);
    TriDiagMatrixMTL<double> contTranMatrixOE(xmax);
    TriDiagMatrixMTL<double> contTranMatrix_infty(xmax);
    TriDiagMatrixMTL<double> contTranMatrix_temp(xmax);
    TriDiagMatrix3D<double> prcontMatrix(xmax,Tbar);
  #elif TRANSITION_MATRIX == 2 
    SparseMatrixMTL<double> tranMatrix;
    SparseMatrixMTL<double> contTranMatrix;
    SparseMatrixMTL<double> contTranMatrixOE;
    SparseMatrixMTL<double> contTranMatrix_infty;
    SparseMatrixMTL<double> contTranMatrix_temp;
    SparseMatrix3D<double> prcontMatrix(Tbar);
  #else 
    #error Invalid value for TRANSITION_MATRIX
  #endif

  MatrixMTL<double> temp(xmax);
  VectorMTL<double> profsim(xmax);

  MatrixMTL3D<double> disc_Freq(xmax,xmax,kmax);
  MatrixMTL<double> disc_Freq_OE(xmax,xmax);

  MatrixMTL<double> proftildesMatrix(Tbar+1,xmax);
  VectorMTL<double> difprofit(xmax);
  VectorMTL<double> difprofitpos(xmax);
  MatrixMTL<double> profitPrecomp(xmax);

  if (bound1avg_nonstat!=NULL) delete bound1avg_nonstat;
  if (bound1var_nonstat!=NULL) delete bound1var_nonstat;
  if (bound1pre_nonstat!=NULL) delete bound1pre_nonstat;
  if (bound1por_nonstat!=NULL) delete bound1por_nonstat;

  bound1avg_nonstat = new VectorMTL<double>(xmax);
  bound1var_nonstat = new VectorMTL<double>(xmax);
  bound1pre_nonstat = new VectorMTL<double>(xmax);
  bound1por_nonstat = new VectorMTL<double>(xmax);

  if (bound2avg_nonstat!=NULL) delete bound2avg_nonstat;
  if (bound2var_nonstat!=NULL) delete bound2var_nonstat;
  if (bound2pre_nonstat!=NULL) delete bound2pre_nonstat;
  if (bound2por_nonstat!=NULL) delete bound2por_nonstat;

  bound2avg_nonstat = new VectorMTL<double>(xmax);
  bound2var_nonstat = new VectorMTL<double>(xmax);
  bound2pre_nonstat = new VectorMTL<double>(xmax);
  bound2por_nonstat = new VectorMTL<double>(xmax);

  if (bound3avg_nonstat!=NULL) delete bound3avg_nonstat;
  if (bound3var_nonstat!=NULL) delete bound3var_nonstat;
  if (bound3pre_nonstat!=NULL) delete bound3pre_nonstat;
  if (bound3por_nonstat!=NULL) delete bound3por_nonstat;

  bound3avg_nonstat = new VectorMTL<double>(xmax);
  bound3var_nonstat = new VectorMTL<double>(xmax);
  bound3pre_nonstat = new VectorMTL<double>(xmax);
  bound3por_nonstat = new VectorMTL<double>(xmax);

  double tol1=bounds[1]/(1+bounds[1]);

  bound1sum = 0;
  bound1sqr = 0;
  bound2sum = 0;
  bound2sqr = 0;
  bound3sum = 0;
  bound3sqr = 0;

  double prodsursum=0;
  double prodsursqr=0;
  double conssursum=0;
  double conssursqr=0;
  double totalsursum=0;
  double totalsursqr=0;
  double invsum=0;
  double invsqr=0;

  MatrixMTL<double> *g=NULL;

  int computebound2 = 1;

  // First compute deterministic part 2

  // 1) Precompute the multiplication matrices and compute non-stationary part of term2 sum
  disc_Freq[0].eye();

  int xmax_ext = xmax * int(bounds[7]);

  tildes = sMatrix.getRow(0);
  prices = pMatrix.getRow(0);

  // Initialize term2
  compProf(tildes, prices, term2);
  proftildesMatrix.setRow(0,term2);

  for(int t=0;t<Tbar;t++) {
    rho = rhoMatrix.getRow(t);
    iota = iotaMatrix.getRow(t);
    continuationProb(rho, prcont, condexp);
    tranProb(iota, tranMatrix, contTranMatrix, prcont);

    // Save transition matrix for later
    prcontMatrix[t]=contTranMatrix;

    // Compute discounted frequency
    contTranMatrix*=constants[2];
    disc_Freq[t+1]=disc_Freq[t].dot(contTranMatrix);

    // Compute profits for t+1
    tildes = sMatrix.getRow(t+1);
    prices = pMatrix.getRow(t+1);

    compProf(tildes, prices, proftildes);

    // Save profit for later
    proftildesMatrix.setRow(t+1,proftildes);
 
    term2+=disc_Freq[t+1].dot(proftildes);
  }

  // Compute the discounted frequency for adding OE bounds at the end
  rho = rhoMatrix.getRow(Tbar); // OE rho
  iota = iotaMatrix.getRow(Tbar); // OE iota
  continuationProb(rho, prcont, condexp);
  tranProb(iota, tranMatrix, contTranMatrixOE, prcont);

  iota = 1e20;
  prcont = 1.0;
  tranProb(iota, tranMatrix, contTranMatrix_infty, prcont);

  contTranMatrix_temp=contTranMatrixOE;
  contTranMatrix_temp*=-constants[2];

  contTranMatrix_temp.addDiag(1);

  if(!contTranMatrix_temp.inverse(temp)) {
    cout << "Bound calculation routine error: Cannot invert the discount Matrix" << endl;
    exit(0);
  }

  // Compute more of x-transition matrices for simulation of the markov chain beyond Tbar
  for(int t=Tbar+1;t<kmax;t++) {
    disc_Freq[t]=disc_Freq[t-1].dot(contTranMatrixOE);
    disc_Freq[t]*=constants[2];
  }

  // Discount the stationary frequency // ADD ADDITIONAL Pbeta
  disc_Freq_OE=disc_Freq[Tbar+1].dot(temp);

  // Add the stationary sum of profits at tildes_oe to term2
  term2+=disc_Freq_OE.dot(prof_oe);


  // Compute the parameters for the simulation of OE at the end of the sum
  MatrixMTL<double> Prob(1,xmax);
  double p=0;

  VectorMTL<double> sta_distr(xmax);

  if(control[2]>0) {
    sta_distr = tildes/(control[2]-1);
  } else {
    if(control[1]==2) {
      int time=1;
      MatrixMTL<double> A(xmax,xmax);
      A.eye();

      for(int i=0;i<xmax;i++) {
        Prob[0][i]=A[int(constants[3])][i];
      }
      A=A.dot(contTranMatrix);

      if (bounds[6]<=sum(Prob.getRow(0))) {
        while(time<bounds[5]) {
          Prob.extraRow(1);
          for(int i=0;i<xmax;i++) {		// row time of matrix Prob is a vector with probs of being
            Prob[time][i]=A[int(constants[3])-1][i];	// in each state after time periods in the industry
          }
          A=A.dot(contTranMatrix);

          if (bounds[6]>sum(Prob.getRow(time))) {
            break;
          }
          // Compute prob. to randomize number of entrants if lambda non-integer
          if(ceil(lambdaVector[Tbar])!=floor(lambdaVector[Tbar])) {
            p=(ceil(lambdaVector[Tbar])-lambdaVector[Tbar])/(ceil(lambdaVector[Tbar])-floor(lambdaVector[Tbar]));
          } else {
            p=0;
          }
          if(time==bounds[5]) {
            cout << "Time limit reached when computing distribution to sample states with fixed number of entrants\n";
          }
          time++;
        }
      }
    }
  }

  g = new MatrixMTL<double>(kmax+1,kmax+1);

  for (int i=0; i<=kmax; i++) {
    for (int m=0;m<=kmax;m++) {
      (*g)[i][m]= binopdf(m,i,1-transition[1]);
    }
  }

  VectorMTL<int> bound3_x(xmax);

  // Start simulation
  double boundspre=tol1+1;

  RanGenLib P;
  int n=1;

  while((boundspre>tol1) || 
    (n<=bounds[4])) {

    #ifdef COUT_DEBUG
      cout << "Bound iteration: " << n << ", TOL: "<< boundspre << endl;
    #endif
    #ifdef DEBUG
      *myfile << "Bound iteration: " << n << ", TOL: "<< boundspre << endl;
    #endif


    if (n>bounds[3]) break; 

    VectorMTL<double> term3(xmax);

    // Simulate industry evolution

    // Simulate starting state
    VectorMTL<double> state(xmax);
    state = sMatrix.getRow(0);

    // Compute profits at initial simulated state
    prices=pMatrix.getRow(0);

    double prodsur, conssur;
    VectorMTL<double> ms(xmax), profit_extra(xmax);

    VectorMTL<double> state_extra(xmax), tildes_extra(xmax), pricesOut(xmax);
    state_extra=sMatrix.getRow(0);

    // Add extra firm if fixed number of firms
    if(control[2]>0)
      state_extra[(int) nonstatparams[17]]++;

    compProfAndStats_extra(state_extra,prices,pricesOut,profit_extra,ms,prodsur,conssur);
    double prodsursim=prodsur;
    double conssursim=conssur;
    double totalsursim=(prodsur+conssur);

    // Initialize investment 
    double invsim=0;  
    for(int x=0;x<xmax;x++) {
      invsim+=state_extra[x]*(iotaMatrix[0][x]);
    }

    compProf(state, prices, profsim);

    // Create difprofit for this industry path
    proftildes=proftildesMatrix.getRow(0);
    profsim-=proftildes;

    for(int x=0;x<xmax;x++) {
        bound3_x[x]=x;
        bound1sum_sim[x]= MACRO__POSITIVE(profsim[x]);
        bound2sum_sim[x]= MACRO__POSITIVE(profsim[x]);
        bound3sum_sim[x]= MACRO__POSITIVE(profsim[x]);
    }


    // Initialize term3, disc_Freq[0]=I
    term3=proftildes;

    double beta = constants[2];

    // Simulate next states, Tbar+1 in the loop because last state goes forever as oe
    for(int t=1;t<kmax;t++) {
      // Draw new state
      for(int x=0;x<xmax;x++) {
        bound3_x[x]=genDistr(P,contTranMatrix_infty.getRow(bound3_x[x]));
      }

      // Simulate evolution
      if(t>Tbar) {
        tildes.genMarkov(P,contTranMatrixOE,state);
        lambda = lambdaVector[Tbar];
        tildes_extra.genMarkov(P,contTranMatrixOE,state_extra);

        // Sum the investment
        if(nonstatparams[16]>0) {
          if(nonstatparams[16]>t) {
            for(int x=0;x<xmax;x++) {
              invsim+=tildes_extra[x]*(iotaMatrix[Tbar][x]);
            }
          }
        } else {
          for(int x=0;x<xmax;x++) {
            invsim+=tildes_extra[x]*(beta*iotaMatrix[Tbar][x]);
          }
        }
      } else {
        tildes.genMarkov(P,prcontMatrix[t-1],state);
        lambda = lambdaVector[t-1];
        tildes_extra.genMarkov(P,prcontMatrix[t-1],state_extra);

        if(nonstatparams[16]>0) {
          if(nonstatparams[16]>t) {
            for(int x=0;x<xmax;x++) {
              invsim+=tildes_extra[x]*(beta*iotaMatrix[t][x]);
            }
          } 
        } else {
          if(t==kmax-1) {
            for(int x=0;x<xmax;x++) {
              invsim+=tildes_extra[x]*(beta*iotaMatrix[t][x])/(1-constants[2]);
            }
          } else {
            for(int x=0;x<xmax;x++) {
              invsim+=tildes_extra[x]*(beta*iotaMatrix[t][x]);
            }
          }
        }
      }

      int entrants;
      // Simulate entry
      if(control[2]>0) {
        entrants = 0;
      } else {
        if(control[1]==2) {
          double p;
          if(ceil(lambda)!=floor(lambda)) {
            p=(ceil(lambda)-lambda)/(ceil(lambda)-floor(lambda));
          } else {
            p=0;
          }
          if(P.genUniform()>p) {
             entrants=int(ceil(lambda));
          } else {
             entrants=int(floor(lambda));
          }
        } else {
          entrants = P.genPoisson(lambda);
        }
      }

      // Add entry
      tildes[int(constants[3])-1]+=entrants;

      // Compute profits at simulated state
      if(t>Tbar) {
        prices=pMatrix.getRow(Tbar);
      } else {
        prices=pMatrix.getRow(t);
      }

      compProfAndStats_extra(tildes_extra,prices,pricesOut,profit_extra,ms,prodsur,conssur);
      if(nonstatparams[16]>0) {
        if(nonstatparams[16]>t) {
          prodsursim+=prodsur;
          conssursim+=conssur;
          totalsursim+=(prodsur+conssur);
        }  
      } else {
        prodsursim+=beta*prodsur;
        conssursim+=beta*conssur;
        totalsursim+=beta*(prodsur+conssur);
      }

      compProf(tildes, prices, profsim);

      // Add discounted profits times unconditional transition matrix to term3
      term3+=disc_Freq[t].dot(profsim);

      // Create difprofit for this industry path
      if(t<=Tbar) {
        proftildes=proftildesMatrix.getRow(t);
        profsim-=proftildes;
      } else {
        profsim-=prof_oe;
      }

      if(computebound2>0) {
        for(int i=0;i<xmax_ext-1;i++) {
          if(MAX(profsim[i],0)-MAX(profsim[i+1],0)>bounds[9]*proftildes[i]) {
            #ifdef COUT_DEBUG
            cout << "Delta_y not increasing (" << MAX(profsim[i],0)-MAX(profsim[i+1],0) << ") tol=" << bounds[9]*proftildes[i] <<
              ") for t=" << t << " and y=" << i << endl;
            #endif
            #ifdef DEBUG
            *myfile << "Delta_y not increasing (" << MAX(profsim[i],0)-MAX(profsim[i+1],0) << ") tol=" << bounds[9]*proftildes[i] <<
              ") for t=" << t << " and y=" << i << endl;
            #endif
            computebound2 = 0;
            *myfile << tildes << endl;
            *myfile << sMatrix.getRow(t) << endl;
          }
        }
      }

      for(int x=0;x<xmax;x++) {
        // Compute bound1
        int lowx = MAX(x-int(bounds[8])*t,0);
        int highx = MIN(x+int(bounds[8])*t,xmax_ext-1);

        bound1sum_sim[x]+=beta*MACRO__POSITIVE(max(profsim.subVector(lowx,highx)));

        if(computebound2==1) {
          // Compute bound2
          if(int(bounds[8])==1) {
            for(int y=x;y<=x+t;y++) {
              bound2sum_sim[x]+=beta*(*g)[t][y-x]*MAX(profsim[MIN(y,xmax_ext-1)],0);
            }
          } else {
            bound2sum_sim[x]=0;
          }

          // Compute bound3
          bound3sum_sim[x]+=beta*MAX(profsim[bound3_x[x]],0);
        } else {
            bound2sum_sim[x]=0;
            bound3sum_sim[x]=0;
        }
      }

      // Prepare for next iteration
      state=tildes;
      state_extra=tildes_extra;
      beta*=constants[2];
    }

    bound1sum += (bound1sum_sim + term2 - term3);
    bound1sqr += (bound1sum_sim + term2 - term3)*(bound1sum_sim + term2 - term3);

    if(computebound2==1) {
      if(int(bounds[8])==1) {
        bound2sum += (bound2sum_sim + term2 - term3);
        bound2sqr += (bound2sum_sim + term2 - term3)*(bound2sum_sim + term2 - term3);
      }
      bound3sum += (bound3sum_sim + term2 - term3);
      bound3sqr += (bound3sum_sim + term2 - term3)*(bound3sum_sim + term2 - term3);
    }

    if(nonstatparams[16]>0) {
      prodsursim/=nonstatparams[16];
      conssursim/=nonstatparams[16];
      invsim/=nonstatparams[16];
    } else {
      prodsursim+=beta/(1-constants[2])*prodsur;
      conssursim+=beta/(1-constants[2])*conssur;
      totalsursim+=beta/(1-constants[2])*(conssur+prodsur);

    }

    prodsursum+=prodsursim;
    prodsursqr+=prodsursim*prodsursim;
    conssursum+=conssursim;
    conssursqr+=conssursim*conssursim;
    totalsursum+=prodsursim+conssursim;
    totalsursqr+=(prodsursim+conssursim)*(prodsursim+conssursim);

    invsum+=invsim;
    invsqr+=invsim*invsim;
//    cout << invsim << endl << invsum << endl << invsqr << endl;

//    if(n>=bounds[4]) {
      if(n>2) {
      compAvgPre(bound1sum,bound1sqr,n,*bound1avg_nonstat,*bound1pre_nonstat,*bound1var_nonstat);
        if(computebound2!=0) {
          if(int(bounds[8])==1) {
            compAvgPre(bound2sum,bound2sqr,n,*bound2avg_nonstat,*bound2pre_nonstat,*bound2var_nonstat);
          } else {
            *bound2avg_nonstat=0;
            *bound2pre_nonstat=0;
            *bound2var_nonstat=0;
          }
          compAvgPre(bound3sum,bound3sqr,n,*bound3avg_nonstat,*bound3pre_nonstat,*bound3var_nonstat);
        } else {
          *bound2avg_nonstat=0;
          *bound2pre_nonstat=0;
          *bound2var_nonstat=0;

          *bound3avg_nonstat=0;
          *bound3pre_nonstat=0;
          *bound3var_nonstat=0;
        }
        boundspre=max(*bound1pre_nonstat);
        double pretemp;
        if((pretemp=max(*bound2pre_nonstat))>boundspre) boundspre=pretemp;
        if((pretemp=max(*bound3pre_nonstat))>boundspre) boundspre=pretemp;

        compAvgPre(prodsursum,prodsursqr,n,prodsurOut_nonstat,prodsurpre_nonstat,prodsurvar_nonstat);
        compAvgPre(conssursum,conssursqr,n,conssurOut_nonstat,conssurpre_nonstat,conssurvar_nonstat);
        compAvgPre(totalsursum,totalsursqr,n,totalsurOut_nonstat,totalsurpre_nonstat,totalsurvar_nonstat);
        compAvgPre(invsum,invsqr,n,invOut_nonstat,invpre_nonstat,invvar_nonstat);
    }
    n++;
  }

  if(g!=NULL) delete g;
}

CompOE_nonstat::~CompOE_nonstat() {
  if(profitMatrix_global!=NULL) {
    delete profitMatrix_global;
  }
  if(rhoMatrix_global!=NULL) {
    delete rhoMatrix_global;
  }
  if(iotaMatrix_global!=NULL) {
    delete iotaMatrix_global;
  }
  if(lambdaVector_global!=NULL) {
    delete lambdaVector_global;
  }
  if(V_global!=NULL) {
    delete V_global;
  }
  if(s_global!=NULL) {
    delete s_global;
  }
  if(p_global!=NULL) {
    delete p_global;
  }

  if (bound1avg_nonstat!=NULL) delete bound1avg_nonstat;
  if (bound1var_nonstat!=NULL) delete bound1var_nonstat;
  if (bound1pre_nonstat!=NULL) delete bound1pre_nonstat;
  if (bound1por_nonstat!=NULL) delete bound1por_nonstat;

  if (bound2avg_nonstat!=NULL) delete bound2avg_nonstat;
  if (bound2var_nonstat!=NULL) delete bound2var_nonstat;
  if (bound2pre_nonstat!=NULL) delete bound2pre_nonstat;
  if (bound2por_nonstat!=NULL) delete bound2por_nonstat;

  if (bound3avg_nonstat!=NULL) delete bound3avg_nonstat;
  if (bound3var_nonstat!=NULL) delete bound3var_nonstat;
  if (bound3pre_nonstat!=NULL) delete bound3pre_nonstat;
  if (bound3por_nonstat!=NULL) delete bound3por_nonstat;
}
